import system
import util
import expr

class Behaviour(object):
    def __init__(self, name, args):
        self.name = name
        self.args = args

    # behaviours that are logically equivalent return identical string
    # representations
    def __str__(self):
        res = ""
        res += self.name
        if self.args != None:
            res += "("
            res += util.seqString(self.args,",")
            res += ")"
        return res

    # return a new Behaviour object that is the concrete behaviour
    # that this concrete behaviour refines in the given system
    def getConcreteRefines(self, system):
        (absBehaviour, map) = self.findAbsBehaviour(system)

        # absBehaviour is the abstract behaviour specification for us
        # map contains a map from abstract param names to real ones

        # find whether this abstract behaviour refines another
        if absBehaviour in system.refines:
            refines = system.refines[absBehaviour]
            
            # check that the behaviour it refines has been specified
            (absRefines, refMap) = refines.findAbsBehaviour(system)

            # apply 'map' to 'refines'
            newArgs = ()
            if refines.args != None:
                for arg in refines.args:
                    newArgs += ( expr.applyMapToStr(map, arg), )
            else:
                newArgs = None
            return Behaviour(refines.name, newArgs)

    # find the abstract behaviour specification for this behaviour
    # returns (b, map) where b is the abstract behaviour and
    #                  map is a dictionary from names in the abstract
    #                  behaviour specification to those used here
    def findAbsBehaviour(self, system):
        map = {}
        for b in system.behaviours.keys():
            try:
                map = expr.matchVars(b, (self.name, self.args))
                # we've found a match
                return (b, map)
            except expr.MatchError, e:
                # ignore the exception
                ()
        else:
            raise ValueError("No matching behaviour specification for behaviour: %s" % str(self))
        
