import gtk
import pango
import types

# a ossibly cyclic directed graph with labelled edges
# nodes also carry a label
class Graph(object):
    def __init__(self, name):
        self.nodes = []
        self.edges = {}
        self.name = name
        self.numNodes = 0
        self.numEdges = 0
        
    def getName(self):
        return self.name
    
    def setRoot(self, node):
        self.root = root
    
    def addNode(self, nodeData):
        index = len(self.nodes)
        self.nodes.append(nodeData)
        self.numNodes += 1
        return index

    def getNode(self, nodeIndex):
        return self.nodes[nodeIndex]
    
    def addEdge(self, srcNode, label, targetNode):
        if srcNode not in self.edges.keys():
            self.edges[srcNode] = {label: [targetNode]}
        else:
            if label not in self.edges[srcNode].keys():
                self.edges[srcNode][label] = [targetNode]
            else:
                assert targetNode not in self.edges[srcNode][label]
                self.edges[srcNode][label].append(targetNode)
                
        self.numEdges += 1

    def nodeHasEdges(self, node):
        return node in self.edges.keys()

    def getNodeEdges(self, node):
        assert self.nodeHasEdges(node)
        return self.edges[node]

    def getEdges(self):
        return self.edges

    def getNumEdges(self):
        return self.numEdges

    def getNumNodes(self):
        return self.numNodes
    
    def getNodes(self):
        return self.nodes

    def toDotFormat(self, nodesToHighlight = (), edgesToHighlight = ()):
        res = ""

        res += "digraph G {\n"
        for index in range(len(self.nodes)):
            if index in nodesToHighlight:
                fillcolor="#aaeeaa"
                style = "filled"
            else:
                fillcolor="white"
                style = "filled"
            res += "  node [label=\"" + str(self.nodes[index]) + "\", fillcolor=\"" + fillcolor + "\", style=\"" + style + "\"] " + str(index) + ";\n"

        res += "\n"

        for srcNode in self.edges.keys():
            for label in self.edges[srcNode].keys():
                for dstNode in self.edges[srcNode][label]:
                    fontsize="11"
                    if (srcNode,label,dstNode) in edgesToHighlight:
                        color="#00aa00"
                        fontcolor="#00aa00"
                        style = "bold"
                    else:
                        color="black"
                        fontcolor="navy"
                        style = ""
                
                    res += "  " + str(srcNode) + " -> " + str(dstNode) + " [label=\"" + label + "\", color=\"" + color + "\", fontcolor=\"" + fontcolor + "\", style=\"" + style + "\", fontsize=\"" + fontsize + "\"];\n"

        res += "}\n"
        return res


# a sublcass of gtk.TreeView that's good for browsing a Graph that
# represents a possibly cyclic graph.
# we populate the tree-model "on demand" as the user expands different parts
# of the treeview, i.e. the infinite structure is computed lazily, which
# is about the only way to handle this that I can see.
class GraphTreeView(gtk.TreeView):    
    def __init__(self, graph, rootNode, title,
                 nodeSelectedCallback = None, edgeSelectedCallback = None,
                 noneSelectedCallback = None):
        self.graph = graph
        self.title = title
        self.rootNode = rootNode
        self.nodeSelectedCallback = nodeSelectedCallback
        self.edgeSelectedCallback = edgeSelectedCallback
        self.noneSelectedCallback = noneSelectedCallback        
        
        self.model = gtk.TreeStore(object)
        gtk.TreeView.__init__(self, self.model)
        self.connect("row-expanded", self.rowExpanded)
        rootIter = self.model.append(None, (rootNode,))
        if self.graph.nodeHasEdges(rootNode):
            edges = self.graph.getNodeEdges(rootNode)
            for label in edges.keys():
                self.model.append(rootIter, (label,))
                
        self.textCell = gtk.CellRendererText()
        #self.pixbufCell = gtk.CellRendererPixbuf()
        self.column = gtk.TreeViewColumn(title)
        #self.column.pack_start(self.pixbufCell, False)
        self.column.pack_start(self.textCell, False)
        # since we store PyObjects in the model, we need to convert them
        # to text so they can be rendered properly
        self.column.set_cell_data_func(self.textCell, self.textCellDataFunc)
        #self.column.set_cell_data_func(self.pixbufCell, self.pixbufCellDataFunc)
        # get pixbufs for the edge and node types
        #self.edgePixbuf = self.render_icon(gtk.STOCK_GO_FORWARD, gtk.ICON_SIZE_MENU)
        #self.nodePixbuf = self.render_icon(gtk.STOCK_YES, gtk.ICON_SIZE_MENU)
        
        self.append_column(self.column)
        self.get_selection().connect("changed", self.selectionChanged)

    def selectionChanged(self, selection):
        assert selection == self.get_selection()

        (model, iter) = selection.get_selected()
        assert model == self.model
        
        if iter == None:
            if self.noneSelectedCallback is not None:
                self.noneSelectedCallback(self, self.graph)
            return
            
        sel = model.get_value(iter, 0)
        if type(sel) is types.IntType:
            # a node has been selected
            if self.nodeSelectedCallback is not None:
                self.nodeSelectedCallback(self, self.graph, sel)
        else:
            # an edge has been selected            
            assert type(sel) is types.StringType
            if self.edgeSelectedCallback is not None:

                parent = self.model.iter_parent(iter)
                # should definitely have a parent
                assert parent != None
                pNode = model.get_value(parent, 0)
                assert type(pNode) is types.IntType
                self.edgeSelectedCallback(self, self.graph, (pNode, sel))
            
            
    # convert the pyobject in the model into text to be rendered
    def textCellDataFunc(self, treeviewcolumn, cell, model, iter):
        assert treeviewcolumn is self.column
        assert cell is self.textCell
        assert model is self.model
        
        pyobj = model.get_value(iter, 0)
        if type(pyobj) is types.IntType:
            cell.set_property('text', self.graph.getNode(pyobj))
            cell.set_property('style', pango.STYLE_ITALIC)
            cell.set_property('weight', pango.WEIGHT_NORMAL)
            cell.set_property('foreground', None)            
        else:
            assert type(pyobj) is types.StringType
            cell.set_property('text', pyobj)
            cell.set_property('style', pango.STYLE_NORMAL)
            cell.set_property('weight', pango.WEIGHT_BOLD)
            cell.set_property('foreground', 'dark blue')
            
    # convert the pyobject in the model into a pixbuf to be rendered
    def pixbufCellDataFunc(self, treeviewcolumn, cell, model, iter):
        assert treeviewcolumn is self.column
        assert cell is self.pixbufCell
        assert model is self.model
        
        pyobj = model.get_value(iter, 0)
        if type(pyobj) is types.IntType:
            cell.set_property('pixbuf', self.nodePixbuf)
        else:
            assert type(pyobj) is types.StringType
            cell.set_property('pixbuf', self.edgePixbuf)
        
            
    def rowExpanded(self, treeview, iter, path):
        assert self == treeview
        # add children to any of the row's children if necessary
        # this way the tree gets built on-demand and its size will be
        # proportional to the amount explored by the user
        pyobj = self.model.get_value(iter, 0)
        if type(pyobj) is types.StringType:
            # we've expanded an edge label -- its children are nodes
            assert self.model.iter_has_child(iter)
            # add children for each of the node's children

            child = self.model.iter_children(iter)
            while child != None:
                if not self.model.iter_has_child(child):
                    childNode = self.model.get_value(child, 0)
                    assert type(childNode) is types.IntType
                    if self.graph.nodeHasEdges(childNode):
                        childEdgeLabels = self.graph.getNodeEdges(childNode).keys()

                        for cEdgeLabel in childEdgeLabels:
                            self.model.append(child, (cEdgeLabel,))
                child = self.model.iter_next(child)
                
        else:
            assert type(pyobj) is types.IntType
            # we've expanded a node -- its children are edges
            node = pyobj
            # add children for each of node's children
            if self.graph.nodeHasEdges(node):
                assert self.model.iter_has_child(iter)

                child = self.model.iter_children(iter)
                while child != None:
                    if not self.model.iter_has_child(child):
                        childLabel = self.model.get_value(child, 0)
                        assert type(childLabel) is types.StringType
                        
                        childChildNodes = self.graph.getNodeEdges(node)[childLabel]
                        for ccNode in childChildNodes:
                            self.model.append(child, (ccNode,))
                    child = self.model.iter_next(child)
                
                    
        
        

    
        
