import util
import gtk
from graph import GraphTreeView
from dotrenderer import DotRenderer
import util

class GraphExplorer(object):

        
    def __init__(self, graph, root):
        self.graph = graph
        self.dotRenderer = None
        try:
            self.dotRenderer = DotRenderer()
        except Exception, e:
            self.dotRenderer = None

        
        name = graph.getName()
        (self.frame, vbox) = util.makeVBoxInFrame(name,  "Explore %s   (%d nodes, %d edges)" % (name, graph.getNumNodes(), graph.getNumEdges()))

        self.renderISMControlsBox = gtk.HBox()
        self.renderISMControlsBox.set_spacing(util.HSPACING)
        
        self.sep = 0.1
        self.sepEntry = gtk.SpinButton(None, 0.0, 2)
        self.sepEntry.set_range(0, 10)
        self.sepEntry.set_value(self.sep)
        self.sepEntry.set_update_policy(gtk.UPDATE_IF_VALID)
        self.sepEntry.set_numeric(True)
        self.sepEntry.connect("changed", self.sepEntryChanged)
        self.sepEntry.set_increments(0.05, 0.5)
        sepLabel = gtk.Label("Node Spacing: ")
        sepBox = gtk.HBox()
        sepBox.pack_start(sepLabel, False, False, 0)
        sepBox.pack_start(self.sepEntry, False, False, 0)
        
        self.updateISM = False
        self.updateISMCheck = gtk.CheckButton("_Render Graph")
        self.updateISMCheck.set_active(self.updateISM)
        self.updateISMCheck.connect("toggled", self.updateISMToggled)

        self.renderISMControlsBox.pack_start(self.updateISMCheck, False, False, 0)
        self.renderISMControlsBox.pack_start(sepBox, False, False, 20)
        
        vbox.pack_start(self.renderISMControlsBox, False, False, 0)
        self.hpane = gtk.HPaned()
        vbox.pack_start(self.hpane)
        
        self.nodeSelectedPixbufs = {}
        self.edgeSelectedPixbufs = {}
        self.noneSelectedPixbuf = None
        
        self.gView = GraphTreeView(graph, root, "Tree View", self.nodeSelected,
                                   self.edgeSelected, self.noneSelected)
        self.gViewContainer = gtk.ScrolledWindow()
        self.gViewContainer.set_policy(gtk.POLICY_AUTOMATIC,
                                       gtk.POLICY_AUTOMATIC)
        self.gViewContainer.set_border_width(util.BORDER_WIDTH)        
        self.gViewContainer.add(self.gView)
        self.hpane.pack1(self.gViewContainer, True)
        self.imageContainer = gtk.ScrolledWindow()
        self.imageContainer.set_policy(gtk.POLICY_AUTOMATIC,
                                       gtk.POLICY_AUTOMATIC)
        self.imageContainer.set_border_width(util.BORDER_WIDTH)
        self.imageViewport = gtk.Viewport()
        self.imageContainer.add(self.imageViewport)
        
        if self.dotRenderer != None:
            self.image = gtk.Image()
            self.imageViewport.add(self.image)
        else:
            
            label = gtk.Label("Graph renderer unavailable. \nRendering disabled.")
            label.set_line_wrap(True)
            self.imageViewport.add(label)
            self.sepEntry.set_sensitive(False)
            self.updateISMCheck.set_sensitive(False)
            
        self.hpane.pack2(self.imageContainer, True)
        self.widget = self.frame

    def updateISMToggled(self, button):
        assert button == self.updateISMCheck

        self.updateISM = button.get_active()

        if self.updateISM:
            # pretend the user has changed their selection to update the
            # display
            self.gView.selectionChanged(self.gView.get_selection())
            self.sepEntry.set_sensitive(False)
        else:
            self.image.set_from_pixbuf(None)
            self.sepEntry.set_sensitive(True)
            
    def sepEntryChanged(self, sepEntry):
        assert sepEntry == self.sepEntry
        assert not self.updateISM
        
        self.sep = sepEntry.get_value()
        
        self.noneSelectedPixbuf = None
        self.nodeSelectedPixbufs.clear()
        self.edgeSelectedPixbufs.clear()
        
    def noneSelected(self, gView, graph):
        assert graph is self.graph
        assert gView is self.gView

        if self.updateISM and self.dotRenderer != None:
            if self.noneSelectedPixbuf is None:
                dotFormat = graph.toDotFormat()
                self.noneSelectedPixbuf = self.dotRenderer.toPixbuf(dotFormat,
                                                                    self.sep)
                
            self.image.set_from_pixbuf(self.noneSelectedPixbuf)
        
    def nodeSelected(self, gView, graph, node):
        assert graph is self.graph
        assert gView is self.gView

        if self.updateISM and self.dotRenderer != None:        
            if node not in self.nodeSelectedPixbufs.keys():
                # draw the pixbuf highlighting this node
                dotFormat = graph.toDotFormat([node])                
                pixbuf = self.dotRenderer.toPixbuf(dotFormat, self.sep)
                self.nodeSelectedPixbufs[node] = pixbuf
            else:
                pixbuf = self.nodeSelectedPixbufs[node]
                
            self.image.set_from_pixbuf(pixbuf)


    def edgeSelected(self, gView, graph, edge):
        assert graph is self.graph
        assert gView is self.gView

        if self.updateISM and self.dotRenderer != None:
            if edge not in self.edgeSelectedPixbufs.keys():
        
                (node, label) = edge
                highlight = []
                dests = graph.getNodeEdges(node)[label]
                for d in dests:
                    highlight.append((node,label,d))
                dotFormat = graph.toDotFormat((), highlight)
                pixbuf = self.dotRenderer.toPixbuf(dotFormat, self.sep)
                self.edgeSelectedPixbufs[edge] = pixbuf
            else:
                pixbuf = self.edgeSelectedPixbufs[edge]
        
            self.image.set_from_pixbuf(pixbuf)
    

    def getWidget(self):
        return self.widget
