import pygtk
import gtk
import gobject
import pango
import sys
import traceback

if gtk.pygtk_version < (2,6,0) or gtk.gtk_version < (2,6,0):
    raise ImportError("PyGTK 2.6.0 and GTK 2.6.0 required")

import util

class LogWindow(object):

    def __init__(self, name):
        self.name = name
        (self.frame, self.vbox) = util.makeVBoxInFrame(name, None)
        self.textview = gtk.TextView()
        self.textview.set_editable(False)
        self.textview.set_wrap_mode(gtk.WRAP_NONE)
        #self.textview.set_left_margin(BORDER_WIDTH)
        self.textview.set_left_margin(util.BORDER_WIDTH)
        # use a monospace font for logging for now
        self.textview.modify_font(pango.FontDescription("Monospace"))
        self.textbuffer = self.textview.get_buffer()
        self.textbuffer.set_text("")
        self.scrolledwindow = gtk.ScrolledWindow(None, None)
        self.scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC,
                                       gtk.POLICY_AUTOMATIC)
        self.scrolledwindow.add(self.textview)
        self.vbox.pack_start(self.scrolledwindow, True, True, 0)
        self.endMark = self.textbuffer.create_mark(None,
                                                   self.textbuffer.get_end_iter(),
                                                   False)

        
        # tracks the number of times performAndLog has been called recursively
        # so we can indent the log output appropriately
        self.callCount = 0

        self.errorTag = self.textbuffer.create_tag(None, foreground="red", foreground_set=True, weight=pango.WEIGHT_BOLD, weight_set=True)
        self.successTag = self.textbuffer.create_tag(None, foreground="dark green", foreground_set=True, weight=pango.WEIGHT_BOLD, weight_set=True)

    def getWidget(self):
        return self.frame

    def log(self, string, lock, tag = None):
        if lock:
            gtk.threads_enter()
            
        endIter = self.textbuffer.get_iter_at_mark(self.endMark)
        if tag != None:
            text = self.textbuffer.insert_with_tags(endIter, string, tag)
        else:
            text = self.textbuffer.insert(endIter, string)
        
        self.textview.scroll_to_mark(self.endMark,0.0)

        if lock:
            gtk.threads_leave()

    # returns true if func(*args) executes without exception, False otherwise
    # and logs what's occurring (desc) and whether it succeeded or failed
    def performAndLog(self, func, args, desc, giveResultOnSuccess = False,
                      lock = False):
        msg = desc.split("\n")
        if len(msg) == 1:
            self.log("  " * self.callCount, lock)
            self.log(desc + " ...\n", lock)
        else:
            for line in msg:
                if line != "":
                    self.log("  " * self.callCount, lock)
                    self.log(line + "\n", lock)
            self.log("  " * self.callCount, lock)
            self.log("...\n", lock)
            
        self.callCount += 1
        
        try:
            result = func(*args)
            self.log("  " * (self.callCount - 1), lock)  
            self.log("Succeeded.\n", lock, self.successTag)
            self.callCount -= 1
            if giveResultOnSuccess:
                return (True, result)
            else:
                return True
        except:
            self.log("  " * (self.callCount - 1), lock)          
            self.log("Failed. ", lock, self.errorTag)
            msg = str(sys.exc_info()[1])
            # add traceback to log output
            #for line in traceback.format_tb(sys.exc_info()[2]):
            #    msg += line
            for line in msg.split("\n"):
                if line != "":
                    self.log("  " * (self.callCount - 1), lock)                
                    self.log(line + "\n", lock, self.errorTag)
            self.callCount -= 1
            return False
        
