# module for handling predicates about systems

import util
import expr
import xml.dom.minidom



class Predicate(object):
    def __init__(self, lhs, rhs, code, precond, cexviewer = None):
        self.lhs = lhs
        self.rhs = rhs
        self.code = code
        self.precond = precond
        self.cexviewer = cexviewer

# represents a set of predicates
class Predicates(object):
    # find the abstract predicate for the given assertion
    # assertion is a pair (name, args), where args is itself a tuple of args
    #
    # returns (p, map) where p is the abstract predicate and
    #                  map is a dictionary from names in the abstract
    #                  predicate to those used in 'assertion'
    def findPredicate(self, assertion):
        map = {}
        for p in self.preds.keys():
            try:
                map = expr.matchVars(p,assertion)
                # we've found a match
                return (p, map)
            except Exception, e:
                # ignore the exception
                ()

        raise ValueError("No matching predicate for: %s" % str(assertion))
                
    # load the given predicates file
    def __init__(self, xmlFile):
        self.preds = {}  # maps from predicate names to Predicate objects
        self.code = ""
        
        def procPredicateNode(self, node):
            
            name = node.attributes["name"].value
            
            if not expr.isValid(name):
                raise ValueError("Invalid predicate name %s" % name)
            
            lhs = node.attributes["lhs"].value
            rhs = node.attributes["rhs"].value
            
            precond = None
            if "precond" in node.attributes.keys():
                precond = node.attributes["precond"].value
            
            code = ""
            cexViewer = ""
            # get the code and cexviewer, if present
            for child in node.childNodes:
                if child.nodeType == child.ELEMENT_NODE:
                    if child.localName == "csp":
                        for codeChild in child.childNodes:
                            if (codeChild.nodeType == codeChild.CDATA_SECTION_NODE):
                                code += codeChild.data
                    elif child.localName == "cexviewer":
                        for codeChild in child.childNodes:
                                    if codeChild.nodeType == codeChild.CDATA_SECTION_NODE:
                                        cexViewer += codeChild.data
            if cexViewer == "":
                cexViewer = None


            self.preds[name] = Predicate(lhs,rhs,code,precond,cexViewer)
            
        # parse the XML    
        xmldoc = xml.dom.minidom.parse(xmlFile).documentElement
        if (xmldoc.nodeType == xmldoc.ELEMENT_NODE and
            xmldoc.localName == "predicates"):
            for child in xmldoc.childNodes:
                if child.nodeType == child.ELEMENT_NODE:
                    if child.localName == "predicate":
                        procPredicateNode(self, child)
                    elif child.localName == "csp":
                        for codeChild in child.childNodes:
                            if (codeChild.nodeType == codeChild.CDATA_SECTION_NODE):
                                self.code += codeChild.data
                        
                    else:
                        raise ValueError("Unexpected node " + child.localName)
        else:
            raise ValueError("Expected root document node of predicates, not " + xmldoc.localName)

            
    def getPredicateNames(self):
        return self.preds.keys()

    # return a pair (dict, code)
    # the keys of the dictionary dict are the predicates, p, in preds.
    # dict[p] is a 3-tuple.
    # dict[p][0] is the precondition for the predicate p as a string
    # dict[p][1] is a refinement assertion for the predicate p.
    # dict[p][2] is a (possibly None) string of Python source that, when
    #    executed, may define the class CounterExampleViewer (conforming
    #    to the API provided by counterexampleviewer.py)
    # code is a string of CSP code needed to implement the given
    # preconditions and refinements
    def buildConcreteRefinementAsserts(self, preds):
        code = self.code + "\n"
        done = ()
        
        dict = {}
        
        for p in preds:
            (absPred, map) = self.findPredicate(p)

            pred = self.preds[absPred]
            
            if pred not in done:
                code += pred.code
                done += ( pred, )
                
            concreteLhs = expr.replaceVars(pred.lhs, map)
            concreteRhs = expr.replaceVars(pred.rhs, map)
            concretePrecond = None
            if pred.precond != None:
                concretePrecond = expr.replaceVars(pred.precond, map)
            
            dict[p] = (concretePrecond, (concreteLhs, concreteRhs),
                       pred.cexviewer)

        return (dict, code)
        
