import threading
import gtk

import ismmaker
import graphexplorer
import logwindow
import util

class ProcessExplorer(threading.Thread):

    # make a process explorer for the given 'process' using the definitions
    # in the file 'cspFile' that represents the system 'system'
    # redrawWindow is a callback to be called when the window is to be redrawn
    def __init__(self, fdr, cspFile, process, system, linesToSections,
                 redrawWindow):
        self.fdr = fdr
        self.cspFile = cspFile
        self.process = process
        self.linesToSections = linesToSections
        self.redrawWindow = redrawWindow
        
        self.name = "Exploring %s" % process
        self.desc = self.name + " in %s" % system
        
        (self.frame, self.vbox) = util.makeVBoxInFrame(self.name, self.desc)


        self.logWindow = logwindow.LogWindow("Log for %s" % self.name)
        self.vboxContents = self.logWindow.getWidget()
        self.vbox.add(self.vboxContents)
        self.errOutputTag = self.logWindow.textbuffer.create_tag(None, foreground="dark gray", foreground_set=True)

        
        threading.Thread.__init__(self)
        
    def getWidget(self):
        return self.frame

    def run(self):
        result =  self.logWindow.performAndLog(
            ismmaker.makeISM,
            (self.fdr, self.cspFile, self.process, self.linesToSections,
             self,
             self),
            "Building ISM for process %s" % self.process,
            True, True)
        
        if result != False:
            (value, res) = result
            assert value == True
            (graph, root) = res
            gtk.threads_enter()
            explorer = graphexplorer.GraphExplorer(graph, root)
            self.vbox.remove(self.vboxContents)
            # the log won't be used again so destroy it
            self.vboxContents.destroy()
            self.vbox.add(explorer.getWidget())
            self.redrawWindow()
            gtk.threads_leave()
            
                
    def getDesc(self):
        return self.desc

    # callback from fdr indicating new stdoutput from FDR
    def newFDROutput(self, text):
        # we ignore stdoutput from FDR for now since it doesn't
        # usually contain anything of interest and contains too much
        # garbage like output to "puts [ cmd ]" commands
        pass

    # callback from fdr indicating new stderr outut from FDR
    def newFDRErrOutput(self, text):
        self.logWindow.log(text, True, self.errOutputTag)

    def makeISMLog(self, msg):
        self.newFDRErrOutput(msg)

        
