import gtk
import pango
import util
from invocationeditor import InvocationEditor

# a simple specialisation of InvocationEditor for eiding code
class SourceCodeEditor(InvocationEditor):

    # create an editor for editing source code.
    #   initialCode is the initial source code we're editing,
    #   commitCallback is a function to call when the user wishes to commit
    #      their edits. it is passed the current value of the edited code.
    #   desc is a description of the code that the user is editing and
    #      is used to label the widget
    def __init__(self, initialCode, commitCallback, desc):

        # the code sits in a TextView
        self.textview = gtk.TextView()
        self.textview.set_editable(True)
        self.textview.set_wrap_mode(gtk.WRAP_NONE)
        self.textview.set_left_margin(util.BORDER_WIDTH)
        
        # use a monospace font for code
        self.textview.modify_font(pango.FontDescription("Monospace"))
        self.textbuffer = self.textview.get_buffer()
        self.textbuffer.set_text(initialCode)

        # we put the textview inside a scrolled window
        self.scrolledwindow = gtk.ScrolledWindow(None, None)
        self.scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC,
                                       gtk.POLICY_AUTOMATIC)
        self.scrolledwindow.add(self.textview)

        InvocationEditor.__init__(self, "_Commit", desc,
                                  commitCallback,
                                  ((None, self.scrolledwindow,
                                   (self.textbuffer, "changed"),
                                    self.getCurrentCode,
                                    self.setCurrentCode),))

    def getCurrentCode(self, scrolledwindow):
        assert scrolledwindow == self.scrolledwindow
        (start, end) = self.textbuffer.get_bounds()
        return self.textbuffer.get_text(start, end)

    def setCurrentCode(self, scrolledwindow, code):
        assert scrolledwindow == self.scrolledwindow        
        self.textbuffer.set_text(code)

            
